# Copyright 2014-2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

from abc import (
    ABCMeta,
    abstractmethod,
    abstractproperty,
    )
import os
import shutil

from mib import (
    utils,
    virt,
    )


class BuildError(Exception):
    """Error class for any build error."""


class Builder:
    """Base class for all builders."""

    __metaclass__ = ABCMeta

    @abstractproperty
    def name(self):
        """Name of the builder."""

    @abstractproperty
    def arches(self):
        """List of support architectures."""

    @abstractmethod
    def build_image(self, params):
        """Builds the image with the given parameters."""

    def populate_parser(self, parser):
        """Add parser options for this builder."""

    def get_contrib_path(self, path):
        """Returns the full path to file in contrib directory for this
        builder."""
        return utils.get_contrib_path(self.name, path)


class VirtInstallBuilder(Builder):
    """Builder that uses virt-install."""

    nic_model = None
    extra_arguments = None
    initrd_inject = None
    install_location = None
    install_cdrom = None

    @abstractproperty
    def os_type(self):
        """OS type for virt-install."""

    @abstractproperty
    def os_variant(self):
        """OS variant for virt-install."""

    @abstractproperty
    def disk_size(self):
        """Size of disk to create for virt-install to use."""

    def full_name(self, params):
        """Return the name of the first part of the generated image."""
        return '%s-%s' % (self.name, params.arch)

    def modify_mount(self, mount_path):
        """Allows modification of the files before the final image
        is generated."""

    def build_image(self, params):
        """Builds the image with virt-install."""
        output = params.output
        interface = params.interface
        ram = params.ram
        vcpus = params.vcpus
        arch = params.arch
        os_type = self.os_type
        os_variant = self.os_variant
        disk_size = self.disk_size
        interface_model = self.nic_model
        install_location = self.install_location
        install_cdrom = self.install_cdrom
        extra_arguments = self.extra_arguments
        initrd_inject = self.initrd_inject

        # Check for valid location
        if install_location is None and install_cdrom is None:
            raise BuildError(
                "Missing install_location or install_cdrom for virt-install.")

        # Naming
        full_name = self.full_name(params)

        # Create work space
        with utils.tempdir() as workdir:
            # virt-install fails to access the directory
            # unless the following permissions are used
            utils.subp(['chmod', '777', workdir])

            # Create the disk, and set the permissions
            # that will allow virt-install to access it
            disk_path = os.path.join(workdir, 'disk.img')
            virt.create_disk(disk_path, disk_size, format='raw')
            utils.subp(['chmod', '777', disk_path])
            disk_str = "path=%s,format=raw" % disk_path

            # Start the installation
            vm_name = 'img-build-%s' % full_name
            network_str = 'bridge=%s' % interface
            if interface_model is not None:
                network_str = '%s,model=%s' % (network_str, interface_model)
            if install_location:
                virt.install_location(
                    vm_name,
                    ram,
                    arch,
                    vcpus,
                    os_type,
                    os_variant,
                    disk_str,
                    network_str,
                    install_location,
                    initrd_inject=initrd_inject,
                    extra_args=extra_arguments)
            else:
                virt.install_cdrom(
                    vm_name,
                    ram,
                    arch,
                    vcpus,
                    os_type,
                    os_variant,
                    disk_str,
                    network_str,
                    install_cdrom)

            # Remove the finished installation from virsh
            virt.undefine(vm_name)

            # Mount the disk image
            mount_path = os.path.join(workdir, "mount")
            os.mkdir(mount_path)
            try:
                utils.mount_loop(disk_path, mount_path)

                # Allow the osystem module to install any needed files
                # into the filesystem
                self.modify_mount(mount_path)

                # Create the tarball
                output_path = os.path.join(workdir, "output.tar.gz")
                utils.create_tarball(output_path, mount_path)
            finally:
                utils.umount_loop(disk_path, mount_path)

            # Place in output
            shutil.move(output_path, output)
