# Copyright 2014-2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

import os
import shutil

from mib.builders import (
    BuildError,
    VirtInstallBuilder,
    )


class CentOSBuilder(VirtInstallBuilder):
    """Builds the CentOS image for both amd64 and i386. Uses virt-install
    to perform this installation process."""

    name = "centos"
    arches = ["i386", "amd64"]
    os_type = "linux"
    os_variant = "rhel5"
    disk_size = 5
    nic_model = "virtio"
    install_location = ""

    def full_name(self, params):
        return 'centos%s-%s' % (params.edition, params.arch)

    def populate_parser(self, parser):
        """Add parser options."""
        parser.add_argument(
            '--edition', default='7',
            help="CentOS edition to generate. (Default: 7)")

    def validate_params(self, params):
        """Validates the command line parameters."""
        if params.edition not in ['6', '7']:
            raise BuildError(
                "Unknown CentOS edition: %s." % params.edition)
        if params.edition == '7' and params.arch == 'i386':
            raise BuildError(
                "Cannot generate CentOS 7 for i386, as only amd64 is "
                "supported.")

    def modify_mount(self, mount_path):
        """Install the curtin directory into mount point."""
        path = None
        if self.edition == '6':
            path = self.get_contrib_path('centos6/curtin')
        elif self.edition == '7':
            path = self.get_contrib_path('centos7/curtin')
        if not os.path.exists(path):
            return
        opt_path = os.path.join(mount_path, 'curtin')
        shutil.copytree(path, opt_path)

    def build_image(self, params):
        self.validate_params(params)
        self.edition = params.edition
        if self.edition == '6':
            arch = params.arch
            if arch == 'i386':
                self.install_location = (
                    "http://mirror.centos.org/centos/6/os/i386")
                self.extra_arguments = (
                    "console=ttyS0 ks=file:/centos6-i386.ks text utf8")
                self.initrd_inject = self.get_contrib_path(
                    "centos6/centos6-i386.ks")
            elif arch == 'amd64':
                self.install_location = (
                    "http://mirror.centos.org/centos/6/os/x86_64")
                self.extra_arguments = (
                    "console=ttyS0 ks=file:/centos6-amd64.ks text utf8")
                self.initrd_inject = self.get_contrib_path(
                    "centos6/centos6-amd64.ks")
        elif self.edition == '7':
            self.install_location = (
                "http://mirror.centos.org/centos/7/os/x86_64")
            self.extra_arguments = (
                "console=ttyS0 inst.ks=file:/centos7-amd64.ks text "
                "inst.cmdline inst.headless")
            self.initrd_inject = self.get_contrib_path(
                "centos7/centos7-amd64.ks")
        super(CentOSBuilder, self).build_image(params)
