# Copyright 2014-2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

import os
import shutil

from mib import utils
from mib.builders import (
    BuildError,
    VirtInstallBuilder,
    )


ISOLINUX_CFG = (
    "default text\n"
    "timeout 0\n"
    "\n"
    "label text\n"
    "  kernel vmlinuz\n"
    "  append initrd=initrd.img linux text console=ttyS0 inst.repo=cdrom "
    "inst.ks=cdrom:/ks.cfg inst.cmdline inst.headless\n")


class RHELBuilder(VirtInstallBuilder):
    """Builds the RHEL image for amd64. Uses virt-install
    to perform this installation process."""

    name = "rhel"
    arches = ["amd64"]
    os_type = "linux"
    os_variant = "rhel5"
    disk_size = 5
    nic_model = "virtio"

    def populate_parser(self, parser):
        """Add parser arguments."""
        parser.add_argument(
            '--rhel-iso', required=True,
            help="Path to RHEL installation ISO.")

    def validate_params(self, params):
        """Validates the command line parameters."""
        self.install_cdrom = params.rhel_iso
        if self.install_cdrom is None:
            raise BuildError(
                "RHEL requires the --rhel-iso option.")
        if not os.path.exists(self.install_cdrom):
            raise BuildError(
                "Invalid RHEL iso. File does not exist.")

    def mount_iso(self, workdir, source):
        """Mounts iso in 'iso' directory under workdir."""
        iso_dir = os.path.join(workdir, 'iso')
        os.mkdir(iso_dir)
        utils.subp([
            'mount',
            source,
            iso_dir,
            ])
        return iso_dir

    def umount_iso(self, iso_dir):
        """Unmounts iso at path."""
        utils.subp(['umount', iso_dir])

    def copy_iso(self, workdir, iso_dir):
        """Copies the contents of the iso_dir, into output dir."""
        output = os.path.join(workdir, 'output')
        shutil.copytree(iso_dir, output)
        return output

    def write_ks(self, output_dir):
        """Writes the kickstarter config into the output_dir at 'ks.cfg'."""
        ks_cfg = self.get_contrib_path('rhel7-amd64.ks')
        shutil.copyfile(ks_cfg, os.path.join(output_dir, 'ks.cfg'))

    def set_timeout_zero(self, output_dir):
        """Sets the isolinux.cfg timeout to zero."""
        isolinux_cfg = os.path.join(output_dir, 'isolinux', 'isolinux.cfg')
        with open(isolinux_cfg, 'w') as stream:
            stream.write(ISOLINUX_CFG + '\n')

    def create_iso(self, workdir, source):
        """Creates iso at output, containing files at source."""
        output = os.path.join(workdir, 'output.iso')
        utils.subp([
            'mkisofs',
            '-o', output,
            '-b', 'isolinux/isolinux.bin',
            '-c', 'isolinux/boot.cat',
            '-no-emul-boot',
            '-boot-load-size', '4',
            '-boot-info-table', '-R', '-J', '-v',
            '-T', source
            ])
        utils.subp(['chmod', '777', workdir])
        utils.subp(['chmod', '777', output])
        return output

    def modify_mount(self, mount_path):
        """Install the curtin directory into mount point."""
        path = self.get_contrib_path('curtin')
        if not os.path.exists(path):
            return
        opt_path = os.path.join(mount_path, 'curtin')
        shutil.copytree(path, opt_path)

    def build_image(self, params):
        self.validate_params(params)

        # Create work space
        with utils.tempdir() as workdir:
            # Copy out the contents of the ISO file.
            iso_dir = self.mount_iso(workdir, self.install_cdrom)
            try:
                output_dir = self.copy_iso(workdir, iso_dir)
            finally:
                self.umount_iso(iso_dir)
                shutil.rmtree(iso_dir)

            # Write the kickstarter config.
            self.write_ks(output_dir)

            # Update isolinux to not have a timeout.
            self.set_timeout_zero(output_dir)

            # Create the final ISO for installation.
            try:
                self.install_cdrom = self.create_iso(workdir, output_dir)
            finally:
                shutil.rmtree(output_dir)

            super(RHELBuilder, self).build_image(params)
