# Copyright 2014-2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

import os
import random

from mib import utils


TAP_PREFIX = "vmtap"
TAP_SEARCH_PATH = '/sys/class/net'


class NetworkError(Exception):
    """Exception raise when error occurs creating or destroy network."""


def get_avaliable_tap_name():
    """Gets the next available tap name."""
    tapnum = 0
    tappath = os.path.join(
        TAP_SEARCH_PATH, '%s%d' % (TAP_PREFIX, tapnum))
    while os.path.exists(tappath):
        tapnum += 1
        tappath = os.path.join(
            TAP_SEARCH_PATH, '%s%d' % (TAP_PREFIX, tapnum))
    return '%s%d' % (TAP_PREFIX, tapnum)


def get_random_qemu_mac():
    """Returns a random mac address with QEMU prefix."""
    mac = [
        0x52, 0x54, 0x00,
        random.randint(0x00, 0x7f),
        random.randint(0x00, 0xff),
        random.randint(0x00, 0xff),
        ]
    return ':'.join(map(lambda x: "%02x" % x, mac))


def create_tap(bridge):
    """Creates the tap device on bridge."""
    tap_name = get_avaliable_tap_name()
    owner = utils.get_sudo_user()

    # Create the tap device
    try:
        utils.subp([
            'ip', 'tuntap',
            'add', 'mode', 'tap',
            'user', owner,
            tap_name,
            ])
    except utils.ProcessExecutionError:
        raise NetworkError(
            'Failed to create tap %s for %s.' % (tap_name, owner))

    # Bring the tap device up
    try:
        utils.subp([
            'ip', 'link',
            'set', tap_name, 'up',
            ])
    except utils.ProcessExecutionError:
        delete_tap(tap_name)
        raise NetworkError('Failed to bring up %s.' % tap_name)

    # Add the tap device to the bridge
    try:
        utils.subp([
            'ip', 'link',
            'set', tap_name,
            'master', bridge,
            ])
    except utils.ProcessExecutionError:
        delete_tap(tap_name)
        raise NetworkError('Failed to add tap %s to %s.' % (tap_name, bridge))
    return tap_name


def delete_tap(tap_name):
    """Deletes the tap device."""
    try:
        utils.subp([
            'ip', 'tuntap',
            'del', 'mode', 'tap',
            tap_name,
            ])
    except utils.ProcessExecutionError:
        raise NetworkError('Failed to delete tap %s.' % tap_name)
