# Copyright 2014-2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

from collections import defaultdict
from contextlib import contextmanager
import os
from shutil import rmtree
import subprocess
import sys
import tempfile


def get_contrib_dir():
    """Return path to the contrib directory."""
    contrib_dir = os.environ.get("MIB_CONTRIB_DIR", None)
    if contrib_dir:
        return contrib_dir
    else:
        return "/usr/share/maas-image-builder/contrib"


def get_contrib_path(name, path):
    """Return the full path of the file in the contrib directory."""
    return os.path.join(get_contrib_dir(), name, path)


def get_sudo_user():
    """Gets the name of the user, that launched sudo."""
    if 'SUDO_USER' not in os.environ:
        return 'root'
    return os.environ['SUDO_USER']


def subp(args, data=None, rcs=None, env=None, capture=False, shell=False):
    """Executes a subprocess.

    :param args: command arguments
    :param data: data to send to stdin
    :param rcs: allowed exit codes
    :param env: spawning environment
    :param capture: capture output
    :param shell: execute in shell
    :returns: (out, err) when capture=True
    :raises ProcessExecutionError: error executing process
    """
    if rcs is None:
        rcs = [0]
    try:
        if not capture:
            stdout = None
            stderr = None
        else:
            stdout = subprocess.PIPE
            stderr = subprocess.PIPE
        stdin = subprocess.PIPE
        sp = subprocess.Popen(args, stdout=stdout,
                              stderr=stderr, stdin=stdin,
                              env=env, shell=shell)
        (out, err) = sp.communicate(data)
        if isinstance(out, bytes):
            out = out.decode()
        if isinstance(err, bytes):
            err = err.decode()
    except OSError as e:
        raise ProcessExecutionError(cmd=args, reason=e)
    rc = sp.returncode  # pylint: disable=E1101
    if rc not in rcs:
        raise ProcessExecutionError(stdout=out, stderr=err,
                                    exit_code=rc,
                                    cmd=args)
    if not out and capture:
        out = ''
    if not err and capture:
        err = ''
    return (out, err)


class ProcessExecutionError(IOError):
    """Exception for subprocess."""

    MESSAGE_TMPL = ('%(description)s\n'
                    'Command: %(cmd)s\n'
                    'Exit code: %(exit_code)s\n'
                    'Reason: %(reason)s\n'
                    'Stdout: %(stdout)r\n'
                    'Stderr: %(stderr)r')

    def __init__(self, stdout=None, stderr=None,
                 exit_code=None, cmd=None,
                 description=None, reason=None):
        if not cmd:
            self.cmd = '-'
        else:
            self.cmd = cmd

        if not description:
            self.description = 'Unexpected error while running command.'
        else:
            self.description = description

        if not isinstance(exit_code, int):
            self.exit_code = '-'
        else:
            self.exit_code = exit_code

        if not stderr:
            self.stderr = ''
        else:
            self.stderr = stderr

        if not stdout:
            self.stdout = ''
        else:
            self.stdout = stdout

        if reason:
            self.reason = reason
        else:
            self.reason = '-'

        message = self.MESSAGE_TMPL % {
            'description': self.description,
            'cmd': self.cmd,
            'exit_code': self.exit_code,
            'stdout': self.stdout,
            'stderr': self.stderr,
            'reason': self.reason,
        }
        IOError.__init__(self, message)


@contextmanager
def tempdir(suffix=b'', prefix=b'img-builder-', location=None):
    """Context manager: temporary directory.

    Creates a temporary directory (yielding its path, as `unicode`), and
    cleans it up again when exiting the context.

    The directory will be readable, writable, and searchable only to the
    system user who creates it.
    """
    path = tempfile.mkdtemp(suffix, prefix, location)
    if isinstance(path, bytes):
        path = path.decode(sys.getfilesystemencoding())
    assert isinstance(path, unicode)
    try:
        yield path
    finally:
        rmtree(path, ignore_errors=True)


def kpartx_add(src):
    """Adds partition mappings for src into kpartx."""
    subp(['kpartx', '-a', src])


def kpartx_list(src):
    """Lists the partition mappins for src in kpartx."""
    out, _ = subp(['kpartx', '-l', src], capture=True)
    return [
        '/dev/mapper/%s' % line.split()[0]
        for line in out.splitlines()
        ]


def kpartx_del(src):
    """Removes partition mappings for src from kpartx."""
    subp(['kpartx', '-d', src])


def mount_loop(src, target, idx=0):
    """Mounts the source using loopback onto the target."""
    kpartx_add(src)
    devs = kpartx_list(src)
    subp(['mount', '-o', 'loop', devs[idx], target])


def umount_loop(src, target):
    """Un-mounts the source using loopback."""
    subp(['umount', target])
    kpartx_del(src)


def create_tarball(output, path):
    """Creates a tarball from path and places into output."""
    subp(['tar', 'zcpf', output, '-C', path, '.'])
